# -*- coding: utf-8 -*-

from django.contrib import admin, messages
from django.utils.translation import ugettext_lazy as _

from jmb.core.admin.options import ExtendibleModelAdmin

from acomea.tracerequest import models


class TraceRequestDataAdmin(ExtendibleModelAdmin):
    list_display = (
        'plugin_setting', 'email', 'first_name', 'last_name', 'username',
        'phone', 'auth_trattamento_dati', 'auth_materiale_commerciale', 'auth_societa_terze',
        'utm_campaign', 'utm_source', 'get_confirmation_url'
    )
    advanced_search_fields = (
        ('plugin_setting', 'email', 'first_name', 'last_name'),
        ('username', 'phone', 'auth_trattamento_dati', 'auth_materiale_commerciale', 'auth_societa_terze'),
        ('utm_campaign__icontains:campaign', 'utm_source__icontains:source'),
    )
    readonly_fields = (
        'date_last_modify', 'date_create', 'plugin_setting', 'email', 'first_name', 'last_name',
        'username', 'password', 'phone',
        'auth_trattamento_dati', 'auth_materiale_commerciale', 'auth_societa_terze',
        'utm_campaign', 'utm_source',
    )

    fieldsets = (
        (_('general info'), {'fields': (
            ('date_last_modify', 'date_create'), 'plugin_setting'
        )}),
        (_('user info'), {'fields': (
            ('username', 'password'),
            ('first_name', 'last_name'),
        )}),
        (_('contact info'), {'fields': (
            ('email', 'phone', 'professionale'),
        )}),
        (_('authorization info'), {'fields': (
            ('auth_trattamento_dati', 'auth_materiale_commerciale', 'auth_societa_terze'),
        )}),
    )

    def resend_post_registration_email(self, request, queryset):
        """
        azione per rigenerare ed inviare mail di attivazione
        """
        sent = 0
        for obj in queryset:
            try:
                obj.send_post_registration_email()
                sent += 1
            except:
                messages.error(request, "Email non rigenerata per utente {}".format(obj.email))
        if sent:
            messages.info(request, "{} emails rigenerate correttamente".format(sent))
    resend_post_registration_email.short_description = _("Reinvia email post registrazione")

    actions = [
        'resend_post_registration_email'
    ]


class Gimme5RegistrationDataAdmin(ExtendibleModelAdmin):
    list_display = (
        'plugin_setting', 'email',
        'auth_trattamento_dati',
    )
    advanced_search_fields = (
        ('plugin_setting', 'email'),
        ('auth_trattamento_dati', ),
    )
    readonly_fields = (
        'date_last_modify', 'date_create', 'plugin_setting', 'email',
        'password',
        'auth_trattamento_dati',
    )

    fieldsets = (
        (_('general info'), {'fields': (
            ('date_last_modify', 'date_create'), 'plugin_setting'
        )}),
        (_('user info'), {'fields': (
            ('email', 'password'),
        )}),
        (_('authorization info'), {'fields': (
            ('auth_trattamento_dati'),
        )}),
    )


class EmailMessageAdmin(ExtendibleModelAdmin):
    list_display = (
        'name', 'subject', 'get_edit_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {'fields': ('name', 'subject', 'body',)}),
    )


class PrivacyMessageAdmin(ExtendibleModelAdmin):
    list_display = (
        'name', 'get_edit_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {'fields': ('name', 'body',)}),
    )


admin.site.register(models.TraceRequestData, TraceRequestDataAdmin)
admin.site.register(models.Gimme5RegistrationData, Gimme5RegistrationDataAdmin)
admin.site.register(models.EmailMessage, EmailMessageAdmin)
admin.site.register(models.PrivacyMessage, PrivacyMessageAdmin)
