# -*- coding: utf-8 -*-
from django.conf import settings
from django.db import models
from jmb.core.db.fields import HTMLFieldTinyMCE
from django.utils.translation import ugettext as _

from cms.models.pluginmodel import CMSPlugin

from jmb.core.db.cmsmodels import StatusModel

from acomea.contact.models import Group
from acomea.event.models import ActiveEvent


class TraceRequestSettings(CMSPlugin, StatusModel):
    """
    Setting used by plugin to create form and register contact.
    """
    name = models.CharField(
        max_length=255,
        verbose_name=_('name'),
    )
    password_present = models.BooleanField(
        default=False,
        verbose_name=_('password present'),
    )
    first_name_present = models.BooleanField(
        default=False,
        verbose_name=_('first name present')
    )
    first_name_required = models.BooleanField(
        default=False,
        verbose_name=_('first name required')
    )
    last_name_present = models.BooleanField(
        default=False,
        verbose_name=_('last name present')
    )
    last_name_required = models.BooleanField(
        default=False,
        verbose_name=_('last name required')
    )
    phone_present = models.BooleanField(
        default=False,
        verbose_name=_('phone present')
    )
    phone_required = models.BooleanField(
        default=False,
        verbose_name=_('phone required')
    )
    message_present = models.BooleanField(
        default=False,
        verbose_name=_('message present')
    )
    message_required = models.BooleanField(
        default=False,
        verbose_name=_('message required')
    )
    contact_group = models.ForeignKey(
        Group,
        related_name='trace_request_plugin_settings',
        verbose_name=_('contact group'),
        help_text=_('contact group in which request is saved'),
    )
    has_redirectable_behaviour = models.BooleanField(
        default=True,
        verbose_name=_('has redirect behaviour'),
        help_text=_('if false make the instance not redirectable if user is logged overriding'
                    ' redirect_url_if_registration_and_logged value')
    )
    redirect_url_if_registration_and_logged = models.URLField(
        verbose_name=_('redirect url if registration and logged'),
        blank=True, null=True,
    )
    has_captcha = models.BooleanField(
        default=True,
        verbose_name=_('has captcha')
    )
    # auth_trattamento_dati
    has_auth_trattamento_dati = models.BooleanField(
        default=False,
        verbose_name=_('has auth trattamento dati')
    )
    requires_auth_trattamento_dati = models.BooleanField(
        default=False,
        verbose_name=_('requires auth trattamento dati')
    )
    auth_trattamento_dati_message = models.ForeignKey(
        "PrivacyMessage",
        blank=True, null=True,
        related_name='auth_trattamento_dati_tracerequest_plugins',
        verbose_name=_('auth trattamento dati message'),
    )
    # auth_materiale_commerciale
    has_auth_materiale_commerciale = models.BooleanField(
        default=False,
        verbose_name=_('has auth materiale commerciale')
    )
    requires_auth_materiale_commerciale = models.BooleanField(
        default=False,
        verbose_name=_('requires auth materiale commerciale')
    )
    auth_materiale_commerciale_message = models.ForeignKey(
        "PrivacyMessage",
        blank=True, null=True,
        related_name='auth_materiale_commerciale_tracerequest_plugins',
        verbose_name=_('auth materiale commerciale message'),
    )
    # auth_societa_terze
    has_auth_societa_terze = models.BooleanField(
        default=False,
        verbose_name=_('has auth societa terze')
    )
    requires_auth_societa_terze = models.BooleanField(
        default=False,
        verbose_name=_('requires auth societa terze')
    )
    auth_societa_terze_message = models.ForeignKey(
        "PrivacyMessage",
        blank=True, null=True,
        related_name='auth_societa_terze_tracerequest_plugins',
        verbose_name=_('auth societa terze message'),
    )
    contact_registration_url = models.CharField(
        verbose_name=_('registration url'),
        blank=True, null=True,
        max_length=250,
    )
    registration_thanks_url = models.CharField(
        verbose_name=_('registration thanks url'),
        max_length=250,
    )
    activation_thanks_url = models.CharField(
        verbose_name=_('activation thanks url'),
        max_length=250,
        blank=True, null=True,
    )
    event = models.ForeignKey(
        ActiveEvent,
        related_name='trace_request_plugin_settings',
        null=True, blank=True,
        verbose_name=_('event')
    )
    using = models.CharField(
        max_length=1,
        choices=(('e', _('embedded')), ('s', _('site'))),
        verbose_name=_('using'),
    )
    image_button = models.ImageField(
        blank=True, null=True,
        upload_to='trace_request/',
        verbose_name=_('image button'),
    )
    text_button = models.CharField(
        max_length=25,
        default='Invia',
        verbose_name=_('text button')
    )
    background_color = models.CharField(
        max_length=6,
        null=True, blank=True,
        verbose_name=_('background RGB color'),
    )
    email_sender = models.CharField(
        _('email Sender'),
        max_length=255,
        default=settings.MAILER_DEFAULT_HEADER_SENDER
    )   
    email_receiver = models.EmailField(
        verbose_name=_('email receiver'),
        default="info@acomea.it",
    )
    information_request_email = models.ForeignKey(
        "EmailMessage",
        null=True, blank=True,
        related_name='information_request_trace_request_plugin_settings',
        verbose_name=_('Email contattaci'),
        help_text=_("{{message}} as contact request text")
    )
    request_email = models.ForeignKey(
        "EmailMessage",
        related_name='request_trace_request_plugin_settings',
        verbose_name=_('post registration email'),
        help_text=_("{{confirm_link}} as registration confirm. {{registration_link}} as link to a registration page"),
        blank=True, null=True,
    )
    confirm_email = models.ForeignKey(
        "EmailMessage",
        blank=True, null=True,
        related_name='confirm_trace_request_plugin_settings',
        verbose_name=_('post activation email'),
        help_text=_("{{registration_link}} as link to a registration page")
    )
    template_name = models.CharField(
        max_length=255,
        choices=settings.TRACEREQUEST_CMSPLUGIN_FORM_TEMPLATES,
        default='bootstrap3/tracerequest/cms_plugins/tracerequest_form.html'
    )
    send_post_email = models.BooleanField(
        default=False,
        verbose_name=_('send post email')
    )
    post_email_receiver = models.EmailField(
        verbose_name=_('post email receiver'),
        default="info@acomea.it",
    )
    has_socials = models.BooleanField(
        default=False,
        verbose_name=_('has socials'),
    )
    id_for_anchor = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_('id for anchor'),
        help_text=_("In case the form returns error use this field to create an anchor in the page."
                    "Insert just the id of the element")
    )

    @property
    def has_registration(self):
        if self.password_present:
            return True
        return False

    class Meta:
        app_label = "tracerequest"
        verbose_name = _('trace request')
        verbose_name_plural = _('trace requests')

    def __unicode__(self):
        status_display_map = {0: "off", 1: "on"}
        return u'{0} - {1} - {2}'.format(
            status_display_map[self.status],
            self.name, self.id
        )


class Gimme5RegistrationFormSettings(CMSPlugin, StatusModel):
    """
    Setting used by plugin to create form and register contact.
    """
    name = models.CharField(
        max_length=255,
        verbose_name=_('name'),
    )
    title = models.CharField(
        blank=True, null=True,
        max_length=255,
        verbose_name=_('title'),
    )
    subtitle = models.CharField(
        blank=True, null=True,
        max_length=255,
        verbose_name=_('subtitle'),
    )
    title_on_left_side = models.BooleanField(
        default=False,
        verbose_name=_('title on the left side')
    )
    # SE FALSE SARANNO PRESENTI TUTTI I CAMPI DEL FORM --> REGISTRAZIONE
    # SE TRUE SARANNO PRESENTTI SOLO EMAIL E PASSWORD --> LOGIN
    only_email_and_password_present = models.BooleanField(
        default=False,
        verbose_name=_('only email and password present'),
    )
    has_captcha = models.BooleanField(
        default=True,
        verbose_name=_('has captcha')
    )
    # auth_trattamento_dati
    has_auth_trattamento_dati = models.BooleanField(
        default=False,
        verbose_name=_('has auth trattamento dati')
    )
    requires_auth_trattamento_dati = models.BooleanField(
        default=False,
        verbose_name=_('requires auth trattamento dati')
    )
    auth_trattamento_dati_message = HTMLFieldTinyMCE(
        blank=True, null=True,
        verbose_name=_('auth trattamento dati message'),
    )
    contact_registration_url = models.CharField(
        verbose_name=_('registration url'),
        blank=True, null=True,
        max_length=250,
    )
    registration_thanks_url = models.CharField(
        verbose_name=_('registration thanks url'),
        max_length=250,
    )
    activation_thanks_url = models.CharField(
        verbose_name=_('activation thanks url'),
        max_length=250,
        blank=True, null=True,
    )
    background_image = models.ImageField(
        blank=True, null=True,
        upload_to='trace_request/',
        verbose_name=_('background image'),
    )
    background_color = models.CharField(
        max_length=6,
        null=True, blank=True,
        verbose_name=_('background RGB color'),
    )
    text_button = models.CharField(
        max_length=25,
        default='Provalo gratis',
        verbose_name=_('text button')
    )
    show_as_button = models.BooleanField(
        default=False,
        verbose_name=_('show as button')
    )
    template_name = models.CharField(
        max_length=255,
        choices=settings.TRACEREQUEST_CMSPLUGIN_FORM_TEMPLATES,
        default='gimme5/tracerequest/cms_plugins/gimme5_registration_form.html'
    )
    has_facebook_social = models.BooleanField(
        default=False,
        verbose_name=_('has facebook social'),
    )
    has_google_social = models.BooleanField(
        default=False,
        verbose_name=_('has google social'),
    )
    has_twitter_social = models.BooleanField(
        default=False,
        verbose_name=_('has twitter social'),
    )
    has_linkedin_social = models.BooleanField(
        default=False,
        verbose_name=_('has linkedin social'),
    )
    show_stores_link = models.BooleanField(
        default=False,
        verbose_name=_('show stores link')
    )
    proxy_pass_url = models.URLField(
        blank=True,
        help_text=_('url da anteporre alla pagina nel caso ci sia bisogno di un proxy'),
        verbose_name=_('proxy pass url')
    )
    facebook_app_id = models.CharField(
        max_length=255,
        blank='True',
        verbose_name=_('facebook app id')
    )
    google_app_id = models.CharField(
        max_length=255,
        blank='True',
        verbose_name=_('google app id')
    )

    @property
    def has_registration(self):
        if self.only_email_and_password_present:
            return False
        return True

    class Meta:
        app_label = "tracerequest"
        verbose_name = _('gimme5 registration form')
        verbose_name_plural = _('gimme5 registration forms')

    def __unicode__(self):
        status_display_map = {0: "off", 1: "on"}
        return u'{0} - {1} - {2}'.format(
            status_display_map[self.status],
            self.name, self.id
        )


class FaiComeIlFaiRegistrationFormSettings(CMSPlugin, StatusModel):
    """
    Setting used by plugin to create form and register contact.
    """
    name = models.CharField(
        max_length=255,
        verbose_name=_('name'),
    )
    title = models.CharField(
        blank=True, null=True,
        max_length=255,
        verbose_name=_('title'),
    )
    auth_trattamento_privacy_message = HTMLFieldTinyMCE(
        blank=True, null=True,
        verbose_name=_('auth trattamento privacy message'),
    )
    contact_registration_url = models.CharField(
        verbose_name=_('registration url'),
        blank=True, null=True,
        max_length=250,
    )
    template_name = models.CharField(
        max_length=255,
        choices=settings.TRACEREQUEST_CMSPLUGIN_FORM_TEMPLATES,
        default='faicomeilfai/tracerequest/cms_plugins/faicomeilfai_registration_form.html'
    )
    proxy_pass_url = models.URLField(
        blank=True,
        help_text=_('url da anteporre alla pagina nel caso ci sia bisogno di un proxy'),
        verbose_name=_('proxy pass url')
    )

    class Meta:
        app_label = "tracerequest"
        verbose_name = _('Fai come il Fai registration form')
        verbose_name_plural = _('Fai come il Fai registration forms')

    def __unicode__(self):
        status_display_map = {0: "off", 1: "on"}
        return u'{0} - {1} - {2}'.format(
            status_display_map[self.status],
            self.name, self.id
        )


# Acutally Gimme5 Campaign generic Registration Form Settings
class Gimme5CampaignRegistrationFormSettings(CMSPlugin, StatusModel):
    """
    Setting used by plugin to create form and register contact.
    """
    name = models.CharField(
        max_length=255,
        verbose_name=_('name'),
    )
    title = models.CharField(
        blank=True, null=True,
        max_length=255,
        verbose_name=_('title'),
    )
    code = models.CharField(
        blank=True, null=True,
        max_length=255,
        verbose_name=_('campaign code'),
    )
    auth_trattamento_privacy_message = HTMLFieldTinyMCE(
        blank=True, null=True,
        verbose_name=_('auth trattamento privacy message'),
    )
    contact_registration_url = models.CharField(
        verbose_name=_('registration url'),
        blank=True, null=True,
        max_length=250,
    )
    template_name = models.CharField(
        max_length=255,
        choices=settings.TRACEREQUEST_CMSPLUGIN_GIMME5_CAMPAIGN_FORM_TEMPLATES
    )
    proxy_pass_url = models.URLField(
        blank=True,
        help_text=_('url da anteporre alla pagina nel caso ci sia bisogno di un proxy'),
        verbose_name=_('proxy pass url')
    )

    class Meta:
        app_label = "tracerequest"
        verbose_name = _('Gimme5 Campaign registration form')
        verbose_name_plural = _('Gimme5 Campaign registration forms')

    def __unicode__(self):
        status_display_map = {0: "off", 1: "on"}
        return u'{0} - {1} - {2}'.format(
            status_display_map[self.status],
            self.name, self.id
        )
