# coding utf-8

from django.conf import settings
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django.http import HttpResponseRedirect
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from jmb.core.middleware.force_response import ForceResponse

from acomea.tracerequest.cms.forms import TraceRequestForm, TraceRequestPluginForm, EventTraceRequestPluginForm, \
                                          Gimme5RegistrationForm, Gimme5RegistrationPluginForm,\
                                          FaiComeIlFaiRegistrationForm, FaiComeIlFaiRegistrationPluginForm,\
                                          Gimme5CampaignRegistrationForm, Gimme5CampaignRegistrationPluginForm
from acomea.tracerequest.cms.models import TraceRequestSettings, Gimme5RegistrationFormSettings, FaiComeIlFaiRegistrationFormSettings, Gimme5CampaignRegistrationFormSettings
from acomea.tracerequest.utils import save_as_tracerequest_plugin


class TraceRequestPlugin(CMSPluginBase):

    class Media:
        js = (
            "tracerequest/js/admin_plugin_tracerequest.js",
        )

    form = TraceRequestPluginForm
    module = _('AcomeA Trace Request')
    model = TraceRequestSettings
    name = _('Trace Request')
    admin_preview = False
    render_template = 'bootstrap3/tracerequest/cms_plugins/empty.html'
    fieldsets = (
        (_('general info'), {'fields': (
            ('name', 'status'), 'template_name', 'has_captcha', 'has_socials'
        )}),
        (_('contact/user'), {'fields': (
            'password_present',
            ('has_redirectable_behaviour', 'redirect_url_if_registration_and_logged',),
            ('first_name_present', 'first_name_required',),
            ('last_name_present', 'last_name_required',),
            ('phone_present', 'phone_required',),
            ('message_present', 'message_required',)
        )}),
        (_('group'), {'fields': (
            ('contact_group',),
        )}),
        (_('privacy'), {'fields': (
            # AUTH TRATTAMENTO DATI
            ('has_auth_trattamento_dati', 'requires_auth_trattamento_dati'),
            'auth_trattamento_dati_message',
            'get_auth_trattamento_dati_message_body',
            # AUTH MATERIALE COMMERCIALE
            ('has_auth_materiale_commerciale', 'requires_auth_materiale_commerciale'),
            'auth_materiale_commerciale_message',
            'get_auth_materiale_commerciale_message_body',
            # AUTH SOCIETA TERZE
            ('has_auth_societa_terze',  'requires_auth_societa_terze'),
            'auth_societa_terze_message',
            'get_auth_societa_terze_message_body',
        )}),
        (_('contact registration'), {'fields': (
            ('contact_registration_url',),
        )}),
        (_('thankyou page'), {'fields': (
            ('registration_thanks_url', 'activation_thanks_url',),
            'id_for_anchor'
        )}),
        (_('using'), {'fields': (
            'using',
        )}),
        (_('custom'), {'fields': (
            'image_button',
            'text_button',
            'background_color',
        )}),
        (_('email'), {'fields': (
            ('email_sender', 'email_receiver'),
            ('send_post_email', 'post_email_receiver'),
            'information_request_email', 'request_email', 'confirm_email',
        )}),
    )
    readonly_fields = (
        'get_auth_trattamento_dati_message_body',
        'get_auth_materiale_commerciale_message_body',
        'get_auth_societa_terze_message_body'
    )

    def get_auth_trattamento_dati_message_body(self, obj):
        if obj.auth_trattamento_dati_message:
            return obj.auth_trattamento_dati_message.body
        return ""
    get_auth_trattamento_dati_message_body.short_description = _('auth trattamento dati message')
    get_auth_trattamento_dati_message_body.allow_tags = True

    def get_auth_materiale_commerciale_message_body(self, obj):
        if obj.auth_materiale_commerciale_message:
            return obj.auth_materiale_commerciale_message.body
        return ""
    get_auth_materiale_commerciale_message_body.short_description = _('auth materiale commerciale message')
    get_auth_materiale_commerciale_message_body.allow_tags = True

    def get_auth_societa_terze_message_body(self, obj):
        if obj.auth_societa_terze_message:
            return obj.auth_societa_terze_message.body
        return ""
    get_auth_societa_terze_message_body.short_description = _('auth societa terze message')
    get_auth_societa_terze_message_body.allow_tags = True

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user
        uid = request.GET.get('uid', None)
        # Plugin must not be rendered if not active
        if not instance.status:
            return context
        if instance.has_redirectable_behaviour and instance.has_registration and user.is_authenticated():
            # We redirect an already logged user to MYACOMEA if plugin is used for user registration
            redirect_url = instance.redirect_url_if_registration_and_logged or settings.MYACOMEA_URL
            raise ForceResponse(HttpResponseRedirect(redirect_url))

        # In other cases we need to render the plugin
        else:
            self.render_template = instance.template_name
            if (
                request.method == 'POST' and
                'tracerequest_submit' in request.POST and
                request.POST['tracerequest_submit'] == str(instance.id)
            ):
                form = TraceRequestForm(data=request.POST, plugin_instance=instance, uid=uid)
                if form.is_valid():
                    trace_request_data = form.save()

                    utm_campaign = request.COOKIES.get('utm_campaign', None)
                    utm_source = request.COOKIES.get('utm_source', None)
                    # if utm_campaign or utm_source:
                    trace_request_data.utm_campaign = utm_campaign
                    trace_request_data.utm_source = utm_source
                    trace_request_data.save()

                    if instance.message_present and instance.information_request_email:
                        # invio mail informativa to mail indicata nel plug_in setting
                        trace_request_data.send_information_request_email()
                    if instance.send_post_email and instance.post_email_receiver:
                        # invio mail informativa to mail indicata nel plug_in setting
                        # con specificata l'utenza e la pagina contenete il tracerequest
                        trace_request_data.send_post_email(request.build_absolute_uri())
                    # invio mail a utente con il link per la conferma dela registrazione
                    trace_request_data.send_post_registration_email()
                    if instance.using == "s":
                        raise ForceResponse(HttpResponseRedirect(instance.registration_thanks_url))

            else:
                form = TraceRequestForm(plugin_instance=instance, uid=uid)
            action_url = request.path
            if request.path.startswith('/scopri-i-pir-acomea'):
                action_url = action_url.replace('/scopri-i-pir-acomea', 'https://pir.acomea.it/')
            if request.path.startswith('/news'):
                action_url = '/news/'
            elif request.path.startswith('/diario-del-fondo'):
                action_url = '/diario-del-fondo/'
            context.update({
                'settings': instance,
                'tracerequest_form': form,
                'placeholder': placeholder,
                'action_url': action_url,
                # this key is for engagement for build social url via javascript (engagement.js)
                'engagement_redirect_url': reverse('tracerequest:trace_request_post_social_login'),
                'redirect_url': reverse(
                    'tracerequest:trace_request_post_social_login'
                ) + '?redirect={}&group_id={}'.format(
                    instance.activation_thanks_url, instance.contact_group.id
                )
            })
        return context


class EventTraceRequestPlugin(TraceRequestPlugin):
    form = EventTraceRequestPluginForm
    name = _('Event Trace Request')
    fieldsets = (
        (_('general info'), {'fields': (
            ('name', 'status'), 'template_name', 'has_captcha', 'has_socials'
        )}),
        (_('contact/user'), {'fields': (
            ('first_name_present', 'first_name_required',),
            ('last_name_present', 'last_name_required',),
            ('phone_present', 'phone_required',),
            ('message_present', 'message_required',)
        )}),
        (_('group/event'), {'fields': (
            ('contact_group', 'event',),
        )}),
        (_('privacy'), {'fields': (
            # AUTH TRATTAMENTO DATI
            ('has_auth_trattamento_dati', 'requires_auth_trattamento_dati'),
            'auth_trattamento_dati_message',
            'get_auth_trattamento_dati_message_body',
            # AUTH MATERIALE COMMERCIALE
            ('has_auth_materiale_commerciale', 'requires_auth_materiale_commerciale'),
            'auth_materiale_commerciale_message',
            'get_auth_materiale_commerciale_message_body',
            # AUTH SOCIETA TERZE
            ('has_auth_societa_terze',  'requires_auth_societa_terze'),
            'auth_societa_terze_message',
            'get_auth_societa_terze_message_body',
        )}),
        (_('thankyou page'), {'fields': (
            ('registration_thanks_url',),
        )}),
        (_('using'), {'fields': (
            'using',
        )}),
        (_('custom'), {'fields': (
            'image_button',
            'text_button',
            'background_color',
        )}),
        (_('email'), {'fields': (
            ('email_sender', 'email_receiver'),
            'information_request_email'
        )}),
    )

    def render(self, context, instance, placeholder):
        request = context['request']
        uid = request.GET.get('uid', None)
        # Plugin must not be rendered if not active
        if not instance.status:
            return context
        # In other cases we need to render the plugin
        else:
            self.render_template = instance.template_name
            if (
                request.method == 'POST' and
                'tracerequest_submit' in request.POST and
                request.POST['tracerequest_submit'] == str(instance.id)
            ):
                form = TraceRequestForm(data=request.POST, plugin_instance=instance, uid=uid)
                if form.is_valid():
                    trace_request_data = form.save(commit=False)
                    if instance.message_present and instance.information_request_email:
                        trace_request_data.send_information_request_email()
                    save_as_tracerequest_plugin(trace_request_data.__dict__, send_event=True)
                    trace_request_data.delete()
                    if instance.using == "s":
                        raise ForceResponse(HttpResponseRedirect(instance.registration_thanks_url))
            else:
                form = TraceRequestForm(plugin_instance=instance, uid=uid)
            context.update({
                'settings': instance,
                'tracerequest_form': form,
                'placeholder': placeholder,
                'redirect_url': reverse(
                    'tracerequest:trace_request_post_social_login'
                ) + '?event_id={}&group_id={}'.format(instance.event.id, instance.contact_group.id)
            })
        return context


class Gimme5RegistrationPlugin(CMSPluginBase):

    class Media:
        js = (
            "tracerequest/js/admin_plugin_tracerequest.js",
        )

    form = Gimme5RegistrationPluginForm
    module = _('Gimme5 Registration Form')
    model = Gimme5RegistrationFormSettings
    name = _('Gimme5 Registration Form')
    admin_preview = False
    render_template = 'gimme5/tracerequest/cms_plugins/empty.html'
    fieldsets = (
        (_('general info'), {'fields': (
            ('name', 'status'), 'template_name', 'proxy_pass_url',
            ('title', 'subtitle'), 'title_on_left_side', 'has_captcha',
            ('has_facebook_social', 'facebook_app_id'),
            ('has_google_social', 'google_app_id'),
            ('has_twitter_social', 'has_linkedin_social'),
            'only_email_and_password_present',
            'show_stores_link'
        )}),
        (_('privacy'), {'fields': (
            # AUTH TRATTAMENTO DATI
            ('has_auth_trattamento_dati', 'requires_auth_trattamento_dati'),
            'auth_trattamento_dati_message',
        )}),
        (_('contact registration'), {'fields': (
            ('contact_registration_url',),
        )}),
        (_('thankyou page'), {'fields': (
            ('registration_thanks_url', 'activation_thanks_url',),
        )}),
        (_('custom'), {'fields': (
            'background_image',
            'background_color',
            'text_button',
            'show_as_button',
        )}),
    )

    def render(self, context, instance, placeholder):
        request = context['request']

        # Plugin must not be rendered if not active
        if not instance.status:
            return context
        # In other cases we need to render the plugin
        self.render_template = instance.template_name
        if request.method == 'POST':
            return context

        form = Gimme5RegistrationForm(plugin_instance=instance)
        action_url = request.path
        context.update({
            'settings': instance,
            'gimme5_form': form,
            'placeholder': placeholder,
            'action_url': action_url,
            # this key is for engagement for build social url via javascript (engagement.js)
            'engagement_redirect_url': reverse('tracerequest:trace_request_post_social_login'),
            'redirect_url': reverse(
                'tracerequest:trace_request_post_social_login'
            ) + '?redirect={}'.format(
                instance.activation_thanks_url
            )
        })
        return context


# Acutally Gimme5 Campaign generic Registration Plugin
class FaiComeIlFaiRegistrationPlugin(CMSPluginBase):

    class Media:
        js = (
            "tracerequest/js/admin_plugin_tracerequest.js",
        )

    form = FaiComeIlFaiRegistrationPluginForm
    module = _('Fai come il Fai Registration Form')
    model = FaiComeIlFaiRegistrationFormSettings
    name = _('Fai come il Fai Registration Form')
    admin_preview = False
    render_template = 'gimme5/tracerequest/cms_plugins/empty.html'
    fieldsets = (
        (_('general info'), {'fields': (
            ('name', 'status'), 'template_name', 'proxy_pass_url',
            'title',
        )}),
        (_('privacy'), {'fields': (
            # AUTH TRATTAMENTO DATI
            'auth_trattamento_privacy_message',
        )}),
        (_('contact registration'), {'fields': (
            ('contact_registration_url',),
        )}),
    )

    def render(self, context, instance, placeholder):
        request = context['request']

        # Plugin must not be rendered if not active
        if not instance.status:
            return context
        # In other cases we need to render the plugin
        self.render_template = instance.template_name
        if request.method == 'POST':
            return context

        form = FaiComeIlFaiRegistrationForm(plugin_instance=instance)
        action_url = request.path
        context.update({
            'settings': instance,
            'faicomeilfai_form': form,
            'placeholder': placeholder,
            'action_url': action_url,
        })
        return context


class Gimme5CampaignRegistrationPlugin(CMSPluginBase):

    class Media:
        js = (
            "tracerequest/js/admin_plugin_tracerequest.js",
        )

    form = Gimme5CampaignRegistrationPluginForm
    module = _('Gimme5 Campaign Registration Form')
    model = Gimme5CampaignRegistrationFormSettings
    name = _('Gimme5 Campaign Registration Form')
    admin_preview = False
    render_template = 'gimme5/tracerequest/cms_plugins/empty.html'
    fieldsets = (
        (_('general info'), {'fields': (
            ('name', 'status'), 'template_name', 'proxy_pass_url',
            'title', 'code',
        )}),
        (_('privacy'), {'fields': (
            # AUTH TRATTAMENTO DATI
            'auth_trattamento_privacy_message',
        )}),
        (_('contact registration'), {'fields': (
            ('contact_registration_url',),
        )}),
    )

    def render(self, context, instance, placeholder):
        request = context['request']

        # Plugin must not be rendered if not active
        if not instance.status:
            return context
        # In other cases we need to render the plugin
        self.render_template = instance.template_name
        if request.method == 'POST':
            return context

        form = Gimme5CampaignRegistrationForm(plugin_instance=instance)
        action_url = request.path
        context.update({
            'settings': instance,
            'faicomeilfai_form': form,
            'placeholder': placeholder,
            'action_url': action_url,
        })
        return context


plugin_pool.register_plugin(TraceRequestPlugin)
plugin_pool.register_plugin(EventTraceRequestPlugin)
plugin_pool.register_plugin(Gimme5RegistrationPlugin)
plugin_pool.register_plugin(FaiComeIlFaiRegistrationPlugin)
plugin_pool.register_plugin(Gimme5CampaignRegistrationPlugin)
