# coding: utf-8

from django.conf import settings
from django.contrib.auth.models import User

from acomea.contact.models import Contact, Email, Phone, PhoneCategory
from acomea.contact.utils.mailer import send_invite_to_contact
from acomea.contact.utils.update import set_contact_in_registration
from acomea.profilo.models import AcomeaProfilo, RichiestaAttivazioneCliente
from acomea.profilo.utils import (
    aggiunta_a_gruppo_ar, aggiunta_a_gruppo_lead, aggiunta_a_gruppo_professionali,
    iscrizione_a_newsletter, set_user_as_active, set_bonifica_in_contact_user
)
from acomea.tracerequest.cms.models import TraceRequestSettings


def save_as_registration_plugin(data, user=None):
    """
    Save actions usually performed on acomea.profilo.forms.AcomeaLeadRegistrationForm submit
    """
    email = data.get('email')
    first_name = data.get('first_name').title()
    last_name = data.get('last_name').title()
    professionale = data.get('professionale', False)
    if not user:
        user = User.objects.create(
            email=email,
            first_name=first_name,
            last_name=last_name,
            password=data.get('raw_password'),
            username=data['username'],
            is_active=False,
        )
        # cambio su sso la password usando quella in chiaro; se uso quella cryptata fallirà login
        user.set_password(data.get('raw_password'))

    profilo = AcomeaProfilo.objects.create(
        user=user,
        professionale=professionale,
        iscrizione_alla_newsletter=True,
    )
    RichiestaAttivazioneCliente.objects.create(profilo=profilo)
    # Second saving usually performed by acomea.profilo.plugins.ProfiloRegistrazioneLeadPlugin after form submit
    contact = set_contact_in_registration(user, data)
    iscrizione_a_newsletter(data, user)
    aggiunta_a_gruppo_ar(user=user)
    aggiunta_a_gruppo_lead(user=user)
    if professionale:
        aggiunta_a_gruppo_professionali(user=user)
    # Third saving usually performed when user clicks on registration link in email
    set_user_as_active(user)
    set_bonifica_in_contact_user(user)
    return user


def save_as_tracerequest_plugin(data, send_event=False):
    """
    Save actions usually performed by acomea.contact.forms.tracerequestform after form submit
    """
    email = data.get('email')
    first_name = data.get('first_name').title()
    last_name = data.get('last_name').title()
    auth_trattamento_dati = data.get('auth_trattamento_dati', False) or False
    auth_materiale_commerciale = data.get('auth_materiale_commerciale', None)
    auth_societa_terze = data.get('auth_societa_terze', False) or False
    phone = data.get('phone', None)
    plugin_setting = data.get('plugin_setting_id', None)
    if plugin_setting:
        plugin_setting = TraceRequestSettings.objects.get(id=int(plugin_setting))

    contacts = Contact.objects.filter(emails__email__iexact=email)
    if contacts:
        contact = contacts[0]
        if not contact.user:
            contact.first_name = first_name
            contact.last_name = last_name
            # contact.bonifica = True
            # contact.save()
    else:
        contact = Contact(
            first_name=first_name,
            last_name=last_name,
            trace_request_plugin=plugin_setting,
            bonifica=True,
        )
        contact.save()
        Email.objects.create(email=email, default=True, contact=contact)

    # Salvataggio campi privacy
    contact.auth_trattamento_dati = auth_trattamento_dati
    contact.auth_materiale_commerciale = auth_materiale_commerciale
    contact.auth_societa_terze = auth_societa_terze

    # Aggiornamento contatto
    contact.save()

    contact.assign_campaign_source(data.get('utm_campaign', None), data.get('utm_source', None))

    # Salvataggio telefono
    if phone:
        if not bool(contact.phones.filter(phone=phone).count()):
            phone_category, created = PhoneCategory.objects.get_or_create(name='cellulare')
            Phone.objects.create(
                phone=phone,
                category=phone_category,
                contact=contact
            )

    # Aggiunta a gruppo
    group = plugin_setting.contact_group if plugin_setting else None
    if group:
        contact.groups.add(group)
        # contact = contact.get_reload_from_db()  # to consider signal "contact_group_signal" manipulation

    # Invio invito
    if plugin_setting and plugin_setting.event and send_event:
        send_invite_to_contact(
            contact, plugin_setting.event,
            test=False, is_trace_request=True, continue_if_not_created=True,
            test_in_subject=False, check_can_create=False,
            add_contact_querystring=True
        )
