# coding: utf-8

import datetime

from django.conf import settings
from django.contrib import messages
from django.contrib.auth import authenticate, login
from django.http import HttpResponseRedirect
from django.views.generic import View
from django.utils.http import base36_to_int
from django.utils.translation import ugettext_lazy as _

from acomea.contact.models import Group
from acomea.event.models import Event, Invite
from acomea.tracerequest.models import TraceRequestData
from acomea.tracerequest.utils import save_as_registration_plugin, save_as_tracerequest_plugin


class TraceRequestPostRegistrationView(View):

    def get(self, request, uidb36, token):
        trace_request_id = base36_to_int(uidb36)
        try:
            if not TraceRequestData.objects.filter(pk=trace_request_id, status=1).update(status=0):
                messages.add_message(request, messages.ERROR, _('Invalid link'))
                return HttpResponseRedirect('/')

            trace_request_data = TraceRequestData.objects.get(pk=trace_request_id)
            if trace_request_data.check_token(token, settings.TRACEREQUEST_TOKEN_DAYS_DURATION*24*60*60):
                if trace_request_data.plugin_setting.has_registration:
                    save_as_registration_plugin(trace_request_data.__dict__)
                save_as_tracerequest_plugin(trace_request_data.__dict__)

                # Invio email conferma se impostata
                if trace_request_data.plugin_setting.confirm_email:
                    trace_request_data.send_post_activation_email()
                # Redirect
                redirect_url = trace_request_data.plugin_setting.activation_thanks_url
                if trace_request_data.plugin_setting.has_registration and trace_request_data.raw_password:
                    user = authenticate(
                        username=trace_request_data.username,
                        password=trace_request_data.raw_password,
                    )
                    login(request, user)
                    redirect_url = redirect_url.rstrip('/')
                    redirect_url += "&firstAccess=true" if len(redirect_url.split("?")) > 1 else "/?firstAccess=true"
                else:
                    redirect_url += "&newContact=true" if len(redirect_url.split("?")) > 1 else "?newContact=true"
                trace_request_data.delete()
                response = HttpResponseRedirect(redirect_url)
                kw = {'domain': '.acomea.it'}
                response.delete_cookie('utm_campaign', **kw)
                response.delete_cookie('utm_source', **kw)
                return response
            else:
                messages.add_message(request, messages.ERROR, _('Invalid link, please register again'))
        except Exception:
            messages.add_message(request, messages.ERROR, _('Invalid link, please register again'))
        return HttpResponseRedirect('/')


def trace_request_post_social_login_view(request, *args, **kwargs):
    """
    View usata per segnalare al tracerequest il primo accesso di uno user via social.
    Se il request.user è stato creato nei 60 secondi precedenti a passaggio in questa view,
    è una nuova registrazione
    """
    url = request.GET.get('redirect', '/')
    if hasattr(request.user, 'contact'):
        contact = request.user.contact
        premium = request.GET.get('premium', False)
        group_id = request.GET.get('group_id', None)
        event_id = request.GET.get('event_id', None)
        time_delta = datetime.datetime.now() - request.user.date_joined
        if premium:
            contact.auth_premium = True
        if group_id:
            try:
                group = Group.objects.get(pk=int(group_id))
                contact.groups.add(group)
                contact = contact.get_reload_from_db()  # to consider signal "contact_group_signal" manipulation
            except Group.DoesNotExist:
                pass
        if event_id:
            try:
                event = Event.objects.get(pk=int(event_id))
                contact.bonifica = True
                contact.auth_materiale_commerciale = False
                invite, invite_created = Invite.objects.get_or_create(event=event, contact=contact)
                if invite_created:
                    invite.is_trace_request = True
                    invite.save()
                url = invite.get_page_url()
            except Event.DoesNotExist:
                messages.error(request, 'Errore generico')
        if time_delta.total_seconds() < 60:
            url += '?firstAccessSocial=true'
        contact.save()
    return HttpResponseRedirect(url)
